/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.IsobusDevice;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;

/**
 * Node for the {@link IsobusDevice}
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages({"isobus project taskdata node name device=Device ({0})"})
public class DeviceNode extends AbstractNode {

    private static final String DEVICE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_device.png";

    public DeviceNode(IsobusDevice isobusDevice) {
        super(Children.create(new DeviceChildFactory(isobusDevice), true), Lookups.fixed(isobusDevice));
        setName(Bundle.isobus_project_taskdata_node_name_device(isobusDevice.getDesignator()));
        setIconBaseWithExtension(DEVICE_ICON);

    }

    /**
     * <p>Child factory for {@link IsobusDevice}</p> <p>Possible children are {@link DeviceElement}, {@link DeviceProcessData}, {@link DeviceValuePresentation}
     * and {@link DeviceProperty}</p>
     *
     * @author Gerben Feenstra
     */
    @NbBundle.Messages({
        "isobus project taskdata aggregatenode name device processdata=Device process data",
        "isobus project taskdata aggregatenode name device elements=Device elements",
        "isobus project taskdata aggregatenode name device value presentations=Device value presentations",
        "isobus project taskdata aggregatenode name device properties=Device properties",})
    static class DeviceChildFactory extends ChildFactory<Node> {

        private static final String DEVICE_ELEMENT_AGGREGATE_NODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_device_elements.png";
        private static final String DEVICE_PROCESS_DATA_AGGREGATE_NODE_ICON = "";
        private static final String DEVICE_VALUE_PRESENTATION_AGGREGATE_NODE_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_value_presentations.png";
        private static final String DEVICE_PROPERTY_AGGREGATE_NODE_ICON = "";
        private final IsobusDevice device;

        public DeviceChildFactory(IsobusDevice device) {
            this.device = device;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {

            // DeviceElement (DET)
            if (!device.getDET().isEmpty()) {
                Children deviceElementChildren = Children.create(new AggregateNodeChildFactory<>(device.getDET()), true);
                toPopulate.add(new IsobusAggregateNode(deviceElementChildren, DEVICE_ELEMENT_AGGREGATE_NODE_ICON, Bundle.isobus_project_taskdata_aggregatenode_name_device_elements()));
            }

            // DeviceProcessData (DPD)
            if (!device.getDPD().isEmpty()) {
                Children deviceProcessDataChildren = Children.create(new AggregateNodeChildFactory<>(device.getDPD()), true);
                toPopulate.add(new IsobusAggregateNode(deviceProcessDataChildren, DEVICE_PROCESS_DATA_AGGREGATE_NODE_ICON, Bundle.isobus_project_taskdata_aggregatenode_name_device_processdata()));
            }

            // DeviceValuePresentation (DVP)
            if (!device.getDVP().isEmpty()) {
                Children deviceValuePresentationChildren = Children.create(new AggregateNodeChildFactory<>(device.getDVP()), true);
                toPopulate.add(new IsobusAggregateNode(deviceValuePresentationChildren, DEVICE_VALUE_PRESENTATION_AGGREGATE_NODE_ICON, Bundle.isobus_project_taskdata_aggregatenode_name_device_value_presentations()));
            }

            // Deviceproperty (DPT)
            if (!device.getDPT().isEmpty()) {
                Children devicePropertyChildren = Children.create(new AggregateNodeChildFactory<>(device.getDPT()), true);
                toPopulate.add(new IsobusAggregateNode(devicePropertyChildren, DEVICE_PROPERTY_AGGREGATE_NODE_ICON, Bundle.isobus_project_taskdata_aggregatenode_name_device_properties()));
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }
    }
}
