/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.AllocationTime;
import nl.cloudfarming.client.isobus.model.DeviceElement;
import nl.cloudfarming.client.isobus.model.IsobusProduct;
import nl.cloudfarming.client.isobus.model.ProductAllocation;
import nl.cloudfarming.client.isobus.model.ValuePresentation;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;

/**
 * <p>Child factory for {@link ProductAllocation}</p>
 * <P>Possible children are {@link IsobusProduct}, {@link DeviceElement}, {@link ValuePresentation} and {@link AllocationTime}</p>
 * @author Gerben Feenstra
 */
public class ProductAllocationChildFactory extends ChildFactory<Node> {

    private final ProductAllocation productAllocation;

    public ProductAllocationChildFactory(ProductAllocation productAllocation) {
        this.productAllocation = productAllocation;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {
        //Product (PDT)
        IsobusProduct isobusProduct = (IsobusProduct) productAllocation.getProductIdRef();
        if(isobusProduct != null){
            toPopulate.add(new ProductNode(isobusProduct));
        }
        
        //DeviceElement (DET)
        DeviceElement deviceElement = (DeviceElement) productAllocation.getDeviceElementIdRef();
        if (deviceElement != null) {
            toPopulate.add(new DeviceElementNode(deviceElement));
        }

        //ValuePresentation (VPN)
        ValuePresentation presentation = (ValuePresentation) productAllocation.getValuePresentationIdRef();
        if (presentation != null) {
            toPopulate.add(new ValuePresentationNode(presentation));
        }

        //AllocationTime (ASP)
        AllocationTime allocationTime = productAllocation.getAllocationTimeStamp();
        if (allocationTime != null) {
            toPopulate.add(new AllocationStampNode(allocationTime));
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;

    }
}
