/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.tree;

import java.util.List;
import nl.cloudfarming.client.isobus.model.IsobusProduct;
import nl.cloudfarming.client.isobus.model.ProductGroup;
import nl.cloudfarming.client.isobus.model.ValuePresentation;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;

/**
 * Node for the {@link IsobusProduct} object
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages({"isobus project taskdata node name product=Product ({0})"})
public class ProductNode extends AbstractNode {

    private static final String PRODUCT_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_product.png";

    public ProductNode(IsobusProduct isobusProduct) {
        super(Children.create(new ProductChildFactory(isobusProduct), true), Lookups.fixed(isobusProduct));
        setName(Bundle.isobus_project_taskdata_node_name_product(isobusProduct.getProductDesigNator()));
        setIconBaseWithExtension(PRODUCT_ICON);

    }

    /**
     * <p>Child factory for {@link IsobusProduct}</p> <p>Possible children are {@link ProductGroup}
     * and {@link ValuePresentation}</p>
     *
     * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
     */
    @NbBundle.Messages({"isobus project taskdata node name product group=Product group ({0})"})
    static class ProductChildFactory extends ChildFactory<Node> {

        private IsobusProduct isobusProduct;
        private static final String PRODUCT_GROUP_ICON = "nl/cloudfarming/client/isobus/tree/isobus_taskdata_productgroup.png";

        ProductChildFactory(IsobusProduct isobusProduct) {
            this.isobusProduct = isobusProduct;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            //ProductGroup (PGP)
            ProductGroup productGroup = (ProductGroup) isobusProduct.getProductGroupId();
            if(productGroup != null){
                toPopulate.add(new IsobusCommonNode(Children.LEAF, productGroup, PRODUCT_GROUP_ICON, Bundle.isobus_project_taskdata_node_name_product_group(productGroup.getProductGRoupDesignator())));
            }
            //ValuePresentation (VPN)
            ValuePresentation valuePresentation = (ValuePresentation) isobusProduct.getValuePresentationIdRef();
            if(valuePresentation != null){
                toPopulate.add(new ValuePresentationNode(valuePresentation));
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }
    }
}
